<?php

	/**
	 * Fired during plugin activation
	 *
	 * @link       http://codeboxr.com
	 * @since      1.0.0
	 *
	 * @package    CBXSCRatingReview
	 * @subpackage CBXSCRatingReview/includes
	 */

	/**
	 * Fired during plugin activation.
	 *
	 * This class defines all code necessary to run during the plugin's activation.
	 *
	 * @since      1.0.0
	 * @package    CBXSCRatingReview
	 * @subpackage CBXSCRatingReview/includes
	 * @author     Sabuj Kundu <sabuj@codeboxr.com>
	 */
	class CBXSCRatingReview_Activator {

		/**
		 * Short Description. (use period)
		 *
		 * Long Description.
		 *
		 * @since    1.0.0
		 */
		public static function activate() {
			//check if can activate plugin
			if ( ! current_user_can( 'activate_plugins' ) ) {
				return;
			}


			$plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
			check_admin_referer( "activate-plugin_{$plugin}" );

			//create tables
			CBXSCRatingReview_Activator::createTables();

		}

		/**
		 * Create  necessary tables needed for 'cbxscratingreview'
		 */
		public static function createTables() {
			global $wpdb;

			$charset_collate = $wpdb->get_charset_collate();


			//tables
			$table_rating_log = $wpdb->prefix . 'cbxscratingreview_log';
			$table_rating_avg = $wpdb->prefix . 'cbxscratingreview_log_avg';


			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

			//create rating log table
			$table_rating_log_sql = "CREATE TABLE $table_rating_log (
                          id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
                          post_id bigint(20) unsigned NOT NULL COMMENT 'foreign key of posts table',
                          post_type varchar(50) NOT NULL COMMENT 'post type e.g. post, page, media',
                          user_id bigint(20) unsigned NOT NULL COMMENT 'foreign key of users table',
                          score float NOT NULL DEFAULT '0' COMMENT 'user given rating, can be half i.e. 3.5',   
                          headline varchar(255) NOT NULL DEFAULT '' COMMENT 'review short title',
                          comment text NOT NULL DEFAULT '' COMMENT 'review full desc',                              
                          attachment text NOT NULL DEFAULT '' COMMENT 'photos or video url',                             
                          extraparams text NOT NULL DEFAULT '' COMMENT 'extra parameters for future new fields',  
                          status varchar(20) NOT NULL DEFAULT 0 COMMENT '0 pending, 1 published, 2 unpublished, 3 spam',                          
                          mod_by bigint(20) unsigned NOT NULL DEFAULT '0' COMMENT 'foreign key of user table. who last modify this list',
                          date_created datetime NOT NULL COMMENT 'created date',
                          date_modified datetime DEFAULT NULL COMMENT 'modified date',
                          PRIMARY KEY (id)
                        ) $charset_collate; ";

			dbDelta( $table_rating_log_sql );

			/*review_date datetime NOT NULL COMMENT 'review date,if not given then create date',
			location varchar(255) NOT NULL DEFAULT '' COMMENT 'review location',
			lat float NOT NULL DEFAULT '0' COMMENT 'review location latitude',
			lon float NOT NULL DEFAULT '0' COMMENT 'review location longitude',*/

			//add_by bigint(20) unsigned NOT NULL DEFAULT '0' COMMENT 'foreign key of user table. who added this, if uest zero',

			//create table for rating avg log
			$table_rating_avg_log_sql = "CREATE TABLE $table_rating_avg (
                          id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
                          post_id bigint(20) unsigned NOT NULL COMMENT 'foreign key of posts table',
                          post_type varchar(50) NOT NULL COMMENT 'post type e.g. post, page, media',
                          avg_rating float NOT NULL DEFAULT '0' COMMENT 'user given rating avg',  
                          total_count bigint(20) unsigned NOT NULL DEFAULT '0' COMMENT 'total user rate for this post',
                          rating_stat text NOT NULL DEFAULT '' COMMENT 'statistics about how many user give 1, 2, 3, 4, 5 rating',  
                		  date_created datetime NOT NULL COMMENT 'created date',
                          date_modified datetime DEFAULT NULL COMMENT 'modified date',
                          PRIMARY KEY (id)
                        ) $charset_collate; ";

			dbDelta( $table_rating_avg_log_sql );
		}//end method createTables

	}//end class CBXSCRatingReview_Activator